<?php

/**
 *
 * @link              https://sitespot.dev
 * @package           Bt_Pro
 *
 * @wordpress-plugin
 * Plugin Name:       BB Max Width Columns
 * Plugin URI:        https://sitespot.dev/downloads/beaver-builder-max-width-column/
 * Description:       A quick and easy way to add a max width to Beaver Builder rows and columns.
 * Version:           1.0.4
 * Author:            SiteSpot Dev
 * Author URI:        https://sitespot.dev
 * License:           GPL-2.0+
 * License URI:       https://www.gnu.org/licenses/gpl-3.0.en.html
 * Text Domain:       bt-pro
 * Domain Path:       /languages

  BB Max Width Columns is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  any later version.
   
  BB Max Width Columns is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
   
  You should have received a copy of the GNU General Public License
  along with BB Max Width Columns. If not, see https://www.gnu.org/licenses/gpl-3.0.en.html.

 */


define( 'SS_MWC_PLUGIN_NAME', 'Beaver Builder Max Width Columns' );
define( 'SS_MWC_VERSION', '1.0.4' ); // also change above
define( 'SS_MWC_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
define( 'SS_MWC_PLUGIN_URI', plugins_url('/', __FILE__) );
define( 'SS_MWC_MERCHANT_URL', 'https://sitespot.dev' );
define( 'SS_MWC_STORE_URL', 'https://sitespot.dev' );
define( 'SS_MWC_ITEM_ID', 141076 );
define( 'SS_MWC_ITEM_NAME', 'Beaver Builder Max Width Column' ); 

if( !class_exists( 'SS_mwc_Plugin_Updater' ) ) {
	include( dirname( __FILE__ ) . '/SS_mwc_Plugin_Updater.php' );
}

function ss_mwc_index_updater() {

    $license_key = 'free-download';// free
	  $edd_updater = new SS_mwc_Plugin_Updater( SS_MWC_STORE_URL, __FILE__,
		array(
			'version' => SS_MWC_VERSION,   // current version number
			'license' => $license_key,                      // license key 
			'item_id' => SS_MWC_ITEM_ID,      // ID of the product
			'author'  => 'SiteSpot.DEV',                    // author of this plugin
			'beta'    => false,
      'url'     => 'https://sitespot.dev',
		)
	);

}

add_action( 'admin_init', 'ss_mwc_index_updater', 0 );



if(! class_exists ( 'Ss_Max_Width_Column') && class_exists('FLBuilder'))
{
  class Ss_Max_Width_Column{

    function __construct(){
      //filters
      add_filter( 'fl_builder_register_settings_form' , array($this,'add_max_width_settings'), 220 , 2 );
      add_filter( 'fl_builder_render_css' , array( $this,'bt_max_width_column_css'), 110, 3);
    }


  function add_max_width_settings( $form, $slug ) {
    if ( 'col' === $slug) {
      $form['tabs']['style']['sections']['general']['fields']['bt_stick_side'] = array(
            'type'          => 'button-group',
            'label'         => 'Max Width Column Align',
            'default'       => 'false',
            'help'          => 'Choose an alignment to activate max-width column',
            'options'       => array(
                'false'      => 'Off',
                'right'      => 'Left',
                'left'      => 'Right',
            ),
            'toggle'        => array(
              'right'      => array(
                'fields'        => array( 'bt_column_max_width'),
              ),
              'left'      => array(
                'fields'        => array( 'bt_column_max_width'),
              ),
            ),
          );
      $form['tabs']['style']['sections']['general']['fields']['bt_column_max_width'] = array(
            'type'  => 'unit',
            'label' => 'Column Max Width',
            'units'          => array( 'px', 'vw', '%' , 'vh' ),
            'default_unit' => 'px', // Optional
            'default' => '600',
            'help' => 'Uses CSS calc() to create column padding to simulate max-width. Any padding you add in the advanced tab will reduce the width of your max-width column',
            'responsive'  => true,
          );
    }
    return $form;
  }

  function bt_max_width_column_css($css, $nodes, $global_settings){

    $medium_breakpoint = $global_settings->medium_breakpoint;
    $responsive_breakpoint = $global_settings->responsive_breakpoint;
    
    $bt_mwr_css = "";
      
    foreach ( $nodes['columns'] as $column ) {
      
      $node = $column->node;
      
      if(isset($column->settings->bt_stick_side) && $column->settings->bt_stick_side != 'false')
      {
        $side = $column->settings->bt_stick_side; // its inverted, as thats how the padding will work      
        
        $padding_side =  "padding_".$side;

        $padding_unit = $column->settings->padding_unit;
        $additional_padding = $column->settings->{$padding_side};
        $max_width = $column->settings->bt_column_max_width;
        $max_unit = $column->settings->bt_column_max_width_unit;

        if($additional_padding == "")
          $additional_padding = 0;
        
        $size = $column->settings->size;      
          $bt_mwr_css .= "@media screen and (min-width:".$responsive_breakpoint."px){
            .fl-node-$node>.fl-col-content{
              padding-$side: calc(100% - ".$max_width.$max_unit." + ".$additional_padding.$padding_unit." ) !important;
            }
          }
          ";
        
      }
      
      if(isset($column->settings->bt_column_max_width_medium) && $column->settings->bt_column_max_width_medium !== '')
      {
        
        $side = $column->settings->bt_stick_side; // its inverted, as thats how the padding will work      
        
        $padding_side =  "padding_".$side."_medium";
        
        $padding_unit = $column->settings->padding_unit;
        $additional_padding = $column->settings->{$padding_side};
        $max_width = $column->settings->bt_column_max_width_medium;
        $max_unit = $column->settings->bt_column_max_width_medium_unit;

        if($additional_padding == "")
          $additional_padding = 0;
        
        $size = $column->settings->size_medium;      

        
          $bt_mwr_css .= "@media screen and (min-width:".$responsive_breakpoint."px) and (max-width:".$medium_breakpoint."px){
            .fl-node-$node>.fl-col-content{
              padding-$side: calc(100% - ".$max_width.$max_unit." + ".$additional_padding.$padding_unit." ) !important;
            }
          }
          ";
        

      }
      
      if(isset($column->settings->bt_column_max_width_responsive) && $column->settings->bt_column_max_width_responsive !== '')
      {
        
        $side = $column->settings->bt_stick_side; // its inverted, as thats how the padding will work      
        
        $padding_side =  "padding_".$side."_responsive";
        
        $padding_unit = $column->settings->padding_unit;
        $additional_padding = $column->settings->{$padding_side};
        $max_width = $column->settings->bt_column_max_width_responsive;
        $max_unit = $column->settings->bt_column_max_width_responsive_unit;

        if($additional_padding == "")
          $additional_padding = 0;
        
        $size = $column->settings->size_responsive;      
        
          $bt_mwr_css .= "@media screen and (max-width:".$responsive_breakpoint."px){
            .fl-node-$node>.fl-col-content{
              padding-$side: calc(100% - ".$max_width.$max_unit." + ".$additional_padding.$padding_unit." ) !important;
            }
          }
          ";
        

      }
    }
      
    return $css . $bt_mwr_css;
    
  }

  }
  $btsc = new Ss_Max_Width_Column();
}


